@@ -1,7 +1,7 @@
 use crate::keybound::Id;
 
 pub trait ViewSet<'a, T: Id> {
-    type Iter: 'a+Iterator<Item=T>;
+    type Iter: 'a+DoubleEndedIterator<Item=T>;
 
     fn contains(&self, k: T) -> bool;
     fn len(&self) -> usize;
@@ -10,9 +10,9 @@ pub trait ViewSet<'a, T: Id> {
 }
 
 pub trait ViewAnyToOne<'a, K: Id, V: Id> {
-    type Iter: 'a+Iterator<Item=(K, V)>;
-    type Keys: 'a+Iterator<Item=K>;
-    type Values: 'a+Iterator<Item=V>;
+    type Iter: 'a+DoubleEndedIterator<Item=(K, V)>;
+    type Keys: 'a+DoubleEndedIterator<Item=K>;
+    type Values: 'a+DoubleEndedIterator<Item=V>;
 
     fn get(&self, k: K) -> Option<V>;
     fn contains_key(&self, k: K) -> bool;
@@ -28,10 +28,10 @@ pub trait ViewAnyToOne<'a, K: Id, V: Id> {
 pub trait ViewAnyToSet<'a, K: Id, V: Id> {
     type VMulti: ViewSet<'a, V>;
 
-    type Iter: 'a+Iterator<Item=(K, V)>;
-    type Keys: 'a+Iterator<Item=K>;
-    type Sets: 'a+Iterator<Item=(K, Self::VMulti)>;
-    type Values: 'a+Iterator<Item=V>;
+    type Iter: 'a+DoubleEndedIterator<Item=(K, V)>;
+    type Keys: 'a+DoubleEndedIterator<Item=K>;
+    type Sets: 'a+DoubleEndedIterator<Item=(K, Self::VMulti)>;
+    type Values: 'a+DoubleEndedIterator<Item=V>;
 
     fn get(&'a self, k: K) -> Self::VMulti;
     fn contains_key(&self, k: K) -> bool;
